// bargraph.js

// Load data from CSV file
d3.csv('data/Global_Map_data.csv').then(function (data) {
    createBarGraph(data, 'bargraph');
});

// Function to create bar graph
function createBarGraph(data, containerId) {
    // Set up margin, width, and height
    const margin = { top: 40, right: 20, bottom: 60, left: 70 };
    const width = 300 - margin.left - margin.right;
    const height = 400 - margin.top - margin.bottom;

    // Create SVG container
    const svg = d3.select(`#${containerId}`)
        .attr('width', width + margin.left + margin.right)
        .attr('height', height + margin.top + margin.bottom)
        .append('g')
        .attr('transform', `translate(${margin.left},${margin.top})`);

    // Calculate average values
    const avgDeaths = d3.mean(data, d => +d['People Death New Count']);
    const avgPositiveCases = d3.mean(data, d => +d['People Positive New Cases Count']);

    // Find countries with highest, average, and least values for new deaths
    const highestDeaths = d3.max(data, d => +d['People Death New Count']);
    const highestDeathsCountry = data.find(d => +d['People Death New Count'] === highestDeaths)['Country Short Name'];

    const leastDeaths = d3.min(data, d => +d['People Death New Count']);
    const leastDeathsCountry = data.find(d => +d['People Death New Count'] === leastDeaths)['Country Short Name'];

    // Find countries with highest, average, and least values for new positive cases
    const highestPositiveCases = d3.max(data, d => +d['People Positive New Cases Count']);
    const highestPositiveCasesCountry = data.find(d => +d['People Positive New Cases Count'] === highestPositiveCases)['Country Short Name'];

    const leastPositiveCases = d3.min(data, d => +d['People Positive New Cases Count']);
    const leastPositiveCasesCountry = data.find(d => +d['People Positive New Cases Count'] === leastPositiveCases)['Country Short Name'];

    // Create data array for display
    const displayData = [
        { label: 'Highest Deaths', value: highestDeaths, country: highestDeathsCountry },
        { label: 'Average Deaths', value: avgDeaths, country: 'Average' },
        { label: 'Least Deaths', value: leastDeaths, country: leastDeathsCountry },
        { label: 'Highest Positive Cases', value: highestPositiveCases, country: highestPositiveCasesCountry },
        { label: 'Average Positive Cases', value: avgPositiveCases, country: 'Average' },
        { label: 'Least Positive Cases', value: leastPositiveCases, country: leastPositiveCasesCountry },
    ];

    // Set up scales
    const xScale = d3.scaleBand()
        .domain(displayData.map(d => d.label))
        .range([0, width])
        .padding(0.1);

    const yScale = d3.scaleLinear()
        .domain([0, d3.max(displayData, d => d.value)])
        .range([height, 0]);

    // Create x-axis
    const xAxis = d3.axisBottom(xScale);
    svg.append('g')
        .attr('transform', `translate(0, ${height})`)
        .call(xAxis)
        .selectAll('text')
        .style('text-anchor', 'end')
        .attr('transform', 'rotate(-45)');

    // Create y-axis
    const yAxis = d3.axisLeft(yScale);
    svg.append('g')
        .call(yAxis);

    // Create bars
    svg.selectAll('.bar')
        .data(displayData)
        .enter()
        .append('rect')
        .attr('class', 'bar')
        .attr('x', d => xScale(d.label))
        .attr('y', d => yScale(d.value))
        .attr('width', xScale.bandwidth())
        .attr('height', d => height - yScale(d.value))
        .attr('fill', 'green')
        .on('mouseover', handleMouseOver)
        .on('mouseout', handleMouseOut);

    // Tooltip functions
    function handleMouseOver(d) {
        d3.select(this)
            .attr('fill', 'orange'); // Change color on hover

        const tooltipText = `<strong>${d.label}</strong><br>
                            Value: ${d.value}<br>
                            Country: ${d.country}`;

        showTooltip(tooltipText);
    }

    function handleMouseOut() {
        d3.select(this)
            .attr('fill', 'green'); // Change color back on mouseout

        hideTooltip();
    }

    function showTooltip(text) {
        d3.select('#tooltip')
            .style('opacity', 1)
            .html(text)
            .style('left', d3.event.pageX + 'px')
            .style('top', d3.event.pageY - 28 + 'px');
    }

    function hideTooltip() {
        d3.select('#tooltip').style('opacity', 0);
    }
}
